<?php
/*
 Plugin Name: Reoccuring Day Countdown
 Plugin URI: http://massive-squad.de/2009/03/wp_rodc
 Description: A reminder for a reoccuring day, with a time countdown as widget.
 Author: Gregor Hengst
 Version: 1.0.3
 Author URI: http://www.massive-squad.de
 */
/*  Copyright 2009  Gregor Hengst  (email : grossgeister@gmx.de)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/** Displays and runs the control settings for the widget.
 * Displayed under Desding->Widgets.
 * @return null
 */
function rodc_control()
{
	$option = get_option('rodc_options');
	$old_options = $option;

	//check if something was posted
	if ( $_POST['rodc_submit'] )
	{
		//get the new values
		$option['title'] = strip_tags( stripslashes( $_POST['rodc_title'] ) );
		$option['upcoming_text'] = strip_tags( stripslashes( $_POST['rodc_upcoming_text']));
		$option['arrived_text'] = strip_tags( stripslashes( $_POST['rodc_arrived_text']));
		$option['day'] = strip_tags( stripslashes( $_POST['rodc_day'] ) );
		$option['interval'] = strip_tags( stripslashes( $_POST['rodc_interval'] ) );

		//check if start time is valid and add to options
		$starttime = strip_tags(stripslashes( $_POST['rodc_starttime'] ) );
		list( $start_hour, $start_min ) = split( ':',$starttime );
		$start_hour = intval($start_hour);
		$start_min = intval($start_min);

		if ( $start_hour >= 0 && $start_hour < 24
		&& $start_min >= 0 && $start_min < 60
		&& is_integer( $start_hour ) && is_integer( $start_min ) )
		{
			$option['start_min'] = $start_min;
			$option['start_hour'] = $start_hour;
		}
		else
		{
			$option['start_min'] = 0;
			$option['start_hour'] = 0;
		}

		//check if option to keep config is set and set option
		$keep_config = $_POST['rodc_keep_config'];
		if ( $keep_config != null )
		{
			$option['keep_config'] = true;
		}
		else
		{
			$option['keep_config'] = false;
		}

		//check if option to show countdown is set and set option
		$show_countdown = $_POST['rodc_show_countdown'];
		if ( $show_countdown != null )
		{
			$option['show_countdown'] = true;
		}
		else
		{
			$option['show_countdown'] = false;
		}

		//if ocptions have changed, update options and reset the next-date
		//for recalculation
		if ( $option != $old_options )
		{
			$option['next_date'] = null;
			update_option( 'rodc_options', $option );
		}
	}

	//get values for display
	$rodc_title = $option['title'];
	$rodc_upcoming_text = $option['upcoming_text'];
	$rodc_arrived_text = $option['arrived_text'];
	$rodc_starttime = $option['start_hour'].":".$option['start_min'];
	$rodc_interval = $option['interval'];
	$rodc_day = $option['day'];
	$rodc_keep_config = $option['keep_config'];
	$rodc_show_countdown = $option['show_countdown'];

	//define selection settings for the options
	$interval_0 = $rodc_interval == 0 ? 'selected' : '';
	$interval_1 = $rodc_interval == 1 ? 'selected' : '';
	$interval_2 = $rodc_interval == 2 ? 'selected' : '';
	$interval_3 = $rodc_interval == 3 ? 'selected' : '';
	$day_su = $rodc_day == 0 ? 'selected' : '';
	$day_mo = $rodc_day == 1 ? 'selected' : '';
	$day_tu = $rodc_day == 2 ? 'selected' : '';
	$day_we = $rodc_day == 3 ? 'selected' : '';
	$day_th = $rodc_day == 4 ? 'selected' : '';
	$day_fr = $rodc_day == 5 ? 'selected' : '';
	$day_sa = $rodc_day == 6 ? 'selected' : '';
	$keep_config = $rodc_keep_config ? 'checked' : '';
	$show_countdown = $rodc_show_countdown ? 'checked' : '';

	print <<<END
<div>
	<label for="rodc_title" style="line-height:35px;display:block;">
 		Titel: <input type="text" id="rodc_title" name="rodc_title" value="$rodc_title" />
	</label>
	<label for="rodc_upcoming_text" style="line-height:35px;display:block;">
		Text: <input type="text" id="rodc_upcoming_text" name="rodc_upcoming_text" value="$rodc_upcoming_text" />
	</label>
	<label for="rodc_arrived_text" style="line-height:35px;display:block;">
		Text bei erreichen: <input type="text" id="rodc_arrived_text" name="rodc_arrived_text" value="$rodc_arrived_text" />
	</label>
	<label for="rodc_show_countdown" style="line-height:35px;display:block;">
 		<input type="checkbox" id="rodc_show_countdown" name="rodc_show_countdown" value="true" $show_countdown> Countdown anzeigen
 	</label>
	Startdatum:
	<label for="rodc_starttime" style="line-height:35px;display:block;">
		Beginn (HH:MM, 00:00 bis 23:59): <input type="text" id="rodc_starttime" name="rodc_starttime" value="$rodc_starttime" />
	</label>
	Wiederhole jeden
	<label for="rodc_interval" style="line-height:35px;display:block;">
		<select name="rodc_interval" id="rodc_interval" value="$rodc_interval" size="1">
			<option value="0" $interval_0>1ten</option>
			<option value="1" $interval_1>2ten</option>
			<option value="2" $interval_2>3ten</option>
			<option value="3" $interval_3>4ten</option>
		</select>
	</label>
	<label for="rodc_day" style="line-height:35px;display:block;">
		<select name="rodc_day" id="rodc_day" value="$rodc_day" size="1">
			<option value="1" $day_mo>Montag</option>
			<option value="2" $day_tu>Dienstag</option>
			<option value="3" $day_we>Mittwoch</option>
			<option value="4" $day_th>Donnerstag</option>
			<option value="5" $day_fr>Freitag</option>
			<option value="6" $day_sa>Samstag</option>
			<option value="0" $day_su>Sonntag</option>
		</select>
 	</label>
 	im Monat.
 	<label for="rodc_keep_config" style="line-height:35px;display:block;">
 		<input type="checkbox" id="rodc_keep_config" name="rodc_keep_config" value="true" $keep_config> Einstellungen behalten
 	</label>
 	<input type="hidden" name="rodc_submit" id="widget-submit" value="1" />
</div>
END;
}

/** Calcucates the date of the given interval-th weekday in the given
 * month of the given year.
 * @param $weekday int from 0 to 6, where 0 is Sunday.
 * @param $month int from 1 to 12, where 1 is January
 * @param $year int year
 * @param $interval int 0 to 3 where 0 is the first weekday of the month
 * @return int $interwal-th $weekday of the $month of $year
 */
function get_occurance( $weekday, $month, $year, $interval )
{
	$first_day = mktime( 0, 0, 0, $month, 1, $year );
	while ( date( "w", $first_day ) != $weekday )
	{
		$first_day = $first_day + 24 * 60 * 60;
	}
	$first_day = $first_day + ( $interval * 7 ) * 24 * 60 * 60;

	return $first_day;
}

/** Displays the widget on the site.
 * @param $args wordpress intern argument list for widget display
 * @return null
 */
function rodc_widget( $args )
{
	$day_seconds = 24*60*60;
	$time_2359_seconds = 23 * 60 * 60 + 59 * 60;
	$hour_seconds = 60 * 60;
	$min_seconds = 60;

	extract( $args );
	$option = get_option( 'rodc_options' );

	$current_time = time();

	$rodc_title = $option['title'];
	$rodc_upcoming_text = $option['upcoming_text'];
	$rodc_arrived_text = $option['arrived_text'];
	$next_date = $option['next_date'];
	$start_time_seconds = $hour_seconds * $option['start_hour'] + $min_seconds * $option['start_min'];
	$weekday = $option['day'];
	$interval = $option['interval'];

	//if recalculation is needed
	if ( $next_date == null
	|| ( $next_date + $time_2359_seconds ) < $current_time
	|| ( $next_date - $current_time) > ( 31*$day_seconds ) )
	{
		//generate the occurrance in this month
		$month = date( "n", $current_time );
		$year = date( "Y", $current_time );
		$day = get_occurance( $weekday, $month, $year, $interval );

		//if now is after the occurrance in this month, do the same for next month
		if ( $day < $current_time )
		{
			$month++;
			//jump to following year, if needed
			if ( $month > 12 )
			{
				$month = 1;
				$year++;
			}
			$day = get_occurance( $weekday, $month, $year, $interval );
		}
		//save the date
		$next_date = $day;
		$option['next_date'] = $next_date;
		update_option( 'rodc_options', $option );
	}

	//print widget
	//add bgein time
	$begin = $next_date  + $start_time_seconds;
	echo $before_widget;
	echo $before_title;
	echo $rodc_title;
	echo $after_title;
	if ( $begin > $current_time ) //date is yet to come...
	{
		echo '<p style="text-align:center;">'.$rodc_upcoming_text;
		if ( $option['show_countdown'] )
		{
			//calculate countdown time - maybe not the most efficient algorithm though.
			$diff = $begin - $current_time;
			$days = intval( $diff / $day_seconds );
			$hour = intval( ( $diff % $day_seconds ) / $hour_seconds );
			$mins = intval( ( $diff % $hour_seconds ) / $min_seconds );
			$secs = $diff % $min_seconds;
			echo '<br/>'.$days.'d : '.$hour.'h : '.$mins.'m : '.$secs.'s';
		}
		echo '</p>';
		echo '<p style="text-align:center; font-size:1.1em;"><b>'.date( 'D, d. F Y,  H:i', $begin ).'</b></p>';
	}
	else //date is now
	{
		echo '<p style="text-align:center; font-size:1.2em;">'.$rodc_arrived_text.'</p>';
	}
	echo $after_widget;
}

/**Initialize the plugin.
 * @return null
 */
function rodc_init()
{
	register_sidebar_widget( 'Reoccuring Day Countdown', 'rodc_widget' );
	register_widget_control( 'Reoccuring Day Countdown', 'rodc_control' );
	register_deactivation_hook( __FILE__, 'rodc_unload' );

	$defaultvalues = array (
	'title' => 'Reoccuring Day Countdown',
	'upcoming_text' => 'Next Reoccuring Day in',
	'arrived_text' => 'The Date is today!',
	'start_min' => 0,
	'start_hour' => 0,
	'interval' => 0,
	'day' => 0,
	'next_date' => null,
	'keep_config' => false,
	'show_countdown' => true
	);
	add_option( 'rodc_options', $defaultvalues );
}

/**Unload the plugin.
 * @return unknown_type
 */
function rodc_unload()
{
	remove_action( 'plugins_loaded', 'rodc_init' );

	//if option is not set to keep the config, remove option from database
	$option = get_option( 'rodc_options' );
	if ( !$option['keep_config'] )
	{
		delete_option( 'rodc_options' );
	}

	unregister_sidebar_widget( 'Reoccuring Day Countdown' );
	unregister_widget_control( 'Reoccuring Day Countdown' );
}

//Registers the plugin function for plugin init
add_action( 'plugins_loaded', 'rodc_init' );
?>